// PCM3002 Codec SPI driver
// Juha Niinikoski 18.05.2005
//
// Software loop SPI interface 18.05.2005
// Codec interface 19.05.2005


// PCM3002 SPI signals

#define ML LATD3		// latch
#define MC LATD2		// clock
#define MD LATD1		// data
#define C_RESET LATD0	// codec reset
#define MLDIR TRISD3	// SPI signal direction bits
#define MCDIR TRISD2
#define MDDIR TRISD1
#define C_RESETDIR TRISD0

// PCM3002 register definitions
#define R0	(0<<9)		// Register select bits
#define R1	(1<<9)
#define R2	(2<<9)
#define R3	(3<<9)

// R0, R1
// 0x00=mute, 0x01=-48.16dB, 0xFF=0 dB(default)
// ATT = 20log(AL[7:0]/256) dB  (exept 0xFF = 0 dB)
#define ALR_MASK 0xFF	// DAC attenuation bits mask
#define LDL (1<<8)		// Left DAC attenuation data load control
#define LDR (1<<8)		// Right DAC attenuation data load control

// R2
#define	MUT (1<<0)		// Mute
#define demp_44_1 (0<<1)	// De-emphasis 44.1 kHz
#define demp_OFF (1<<1)	// De-emphasis Off
#define demp_48 (2<<1)	// De-emphasis 48 kHz
#define demp_32 (3<<1)	// De-emphasis 32 kHz
#define OUT_BPZ	(1<<3)	// Set outputs to Vcc/2, bipolar zero
#define IZD		(1<<4)	// Infinite zero detection enable
#define ATC		(1<<5)	// Common attenuation data control, R0 for both
#define PDDA	(1<<6)	// DAC power down
#define BYPS	(1<<7)	// ADC high-pass filter disable(bypass)
#define PDAD	(1<<8)	// ADC power down

//R3
#define LRP		(1<<1)	// LCRIN polarity select, Left = L, Right = H
#define FMT0	(0<<2)	// Format 0 16-bit, DAC right-justified, ADC left-justified
#define FMT1	(1<<2)	// Format 1 20-bit, DAC right-justified, ADC left-justified
#define FMT2	(2<<2)	// Format 2 20-bit, DAC left-justified, ADC left-justified
#define FMT3	(3<<2)	// Format 3 20-bit I2S format
#define LOP		(1<<5)	// Loopback enable


// PCM3002 Register init values
#define R0_INIT	R0 | 0xFF				// L_DAC gain
#define R1_INIT R1 | 0xFF				// R_DAC gain
#define R2_INIT R2 | ATC | demp_OFF		// R0 = LR_DAC gain
#define R3_INIT R3						// default

static unsigned int r0;
static unsigned int r1;
static unsigned int r2;
static unsigned int r3;

// digital resonator
#define m		16384
#define k 		32588			// 400 Hz
#define r_init	-1713

static long X0;
static int X1, X2;

// Init SPI pins

void init_spi(void)
	{
	C_RESET = 0;		// set signals to default state
	ML = 1;
	MC = 0;
	MLDIR = 0;			// change pins to output
	MCDIR = 0;
	MDDIR = 0;
	C_RESETDIR = 0;
	}


// Send 16bits to PCM3002 SPI

void write_spi(unsigned int data)
	{
	char x;

	MC = 0;
	ML = 0;					// start write cycle
	for(x=0; x<16; x++)
		{
		if(data & 0x8000)	// set data
			MD = 1;
		else
			MD = 0;

		data = data << 1;	// addjust data

		MC = 1;				// generate clock
		asm("nop");
		MC = 0;
		}
	asm("nop");
	ML = 1;					// latch
	}

// PCM 3002 codec init

void init_pcm3002(void)
	{
	r0 = R0_INIT;
	write_spi(r0);
	r1 = R1_INIT;
	write_spi(r1);
	r2 = R2_INIT;
	write_spi(r2);
	r3 = R3_INIT;
	write_spi(r3);
	}

// dsPIC30F6012 DCI interface init

void init_dci(void)
	{
// init DCI codec interface for PCM3002 16-bit mode

	DCICON2 = 0x042F;
	RSCON = 0x0003;			// select active slots
	TSCON = 0x0003;
	DCICON1 = 0x8721;		// enable DCI

	TXBUF0 = 0;				// clear fuffers
	TXBUF1 = 0;
	TXBUF2 = 0;
	TXBUF3 = 0;

	X2 = r_init;
	X0 = 0;
	X1 = 0;
	}

// calculate digital resonator (test)
// X0 = k * X1 - X2
// X2 = X1
// X1 = X0

int resonator(void)
	{
	X0 = ((k * X1) >> 15) - X2;
	X2 = X1;
	X1 = X0;

	return X0;
	}

// test
	union
		{
		long l;
		int i[2];
		} out;


// FIR function prototype
void fir( void );

//
// DCI IRQ handler
//
// Performance
// 64MHz osc, 16MHz exec
// IRQ loop & Filters (32 tap) about 10 us
//
void interrupt dci_int(void) @ DCI_VCTR
	{
	DCIIF = 0;
	RD5 = 1;	// indicator

	if(mode == 0)			// signal from ADC
		{
		TXBUF0 = RXBUF0;	// direct out
		TXBUF1 = RXBUF1;

//		TXBUF1 = (int)(sin_table[sin_idx]/2 + (int)RXBUF0/2); // test, sum 240Hz to output
		}

	else
		{
		if(iq_test)			// I / Q signal from internal generator
			{
			TXBUF0 = sin_table[sin_idx];
			TXBUF1 = sin_table[cos_idx];
			}
		else
			{
			fir();			// filter RXBUF 0:1 to TXBUF 0:1
			}
		}

	sin_idx++;
	if(sin_idx > 99)
		sin_idx = 0;

	cos_idx++;
	if(cos_idx > 99)
		cos_idx = 0;

	RD5 = 0;
	}
